@@ -4,17 +4,19 @@ __all__ = [
     "DataPackDirective",
     "ResourcePackDirective",
     "BundleFragmentMixin",
+    "RequireDirective",
+    "SkipDirective",
     "get_builtin_directives",
 ]
 
 
 import io
 from dataclasses import dataclass
-from typing import Dict, Protocol, Type, Union
+from typing import Any, Dict, Protocol, Type, Union
 from zipfile import ZipFile
 
-from beet import DataPack, ResourcePack
-from beet.library.base import NamespaceFile
+from beet import Context, DataPack, ResourcePack
+from beet.library.base import NamespaceFile, NamespaceProxy
 from beet.library.data_pack import (
     Advancement,
     Biome,
@@ -29,6 +31,7 @@ from beet.library.data_pack import (
     FluidTag,
     Function,
     FunctionTag,
+    ItemModifier,
     ItemTag,
     LootTable,
     NoiseSettings,
@@ -71,14 +74,18 @@ class NamespacedResourceDirective:
 
     def __call__(self, fragment: Fragment, assets: ResourcePack, data: DataPack):
         full_name = fragment.expect("full_name")
-        content = fragment.apply_modifier()
+        file_instance: Any = fragment.as_file(self.file_type)
 
-        file_instance = self.file_type(content)
+        pack = assets if self.file_type in assets.namespace_type.field_map else data
+        proxy: NamespaceProxy[Any] = pack[self.file_type]  # type: ignore
 
-        if self.file_type in assets.namespace_type.field_map:
-            assets[full_name] = file_instance
+        if fragment.modifier == "append":
+            current_file = proxy.setdefault(full_name, self.file_type(""))
+            current_file.text += file_instance.text
+        elif fragment.modifier == "merge":
+            proxy.merge({full_name: file_instance})
         else:
-            data[full_name] = file_instance
+            proxy[full_name] = file_instance
 
 
 class BundleFragmentMixin:
@@ -87,13 +94,14 @@ class BundleFragmentMixin:
     def bundle_pack_fragment(self, fragment: Fragment) -> ZipFile:
         """Return a zipfile containing the pack fragment."""
         relative_path = fragment.expect("relative_path")
-        content = fragment.apply_modifier()
+        file_instance = fragment.as_file()
 
         data = io.BytesIO()
         with ZipFile(data, mode="w") as zip_file:
-            zip_file.writestr(relative_path, content)
+            file_instance.dump(zip_file, relative_path)
 
-        return ZipFile(io.BytesIO(data.getvalue()))
+        data.seek(0)
+        return ZipFile(data)
 
 
 @dataclass
@@ -112,8 +120,32 @@ class ResourcePackDirective(BundleFragmentMixin):
         assets.load(self.bundle_pack_fragment(fragment))
 
 
+@dataclass
+class RequireDirective:
+    """Directive that requires a given plugin."""
+
+    ctx: Context
+
+    def __call__(self, fragment: Fragment, assets: ResourcePack, data: DataPack):
+        self.ctx.require(fragment.expect("plugin"))
+
+
+@dataclass
+class SkipDirective:
+    """Directive that ignores the fragment and skips to the next one."""
+
+    def __call__(self, fragment: Fragment, assets: ResourcePack, data: DataPack):
+        fragment.expect()
+
+
 def get_builtin_directives() -> Dict[
-    str, Union[NamespacedResourceDirective, DataPackDirective, ResourcePackDirective]
+    str,
+    Union[
+        NamespacedResourceDirective,
+        DataPackDirective,
+        ResourcePackDirective,
+        SkipDirective,
+    ],
 ]:
     """Return the built-in directives."""
     return {
@@ -139,6 +171,7 @@ def get_builtin_directives() -> Dict[
         "noise_settings":               NamespacedResourceDirective(NoiseSettings),
         "processor_list":               NamespacedResourceDirective(ProcessorList),
         "template_pool":                NamespacedResourceDirective(TemplatePool),
+        "item_modifier":                NamespacedResourceDirective(ItemModifier),
 
         "blockstate":      NamespacedResourceDirective(Blockstate),
         "model":           NamespacedResourceDirective(Model),
@@ -155,5 +188,7 @@ def get_builtin_directives() -> Dict[
 
         "data_pack":     DataPackDirective(),
         "resource_pack": ResourcePackDirective(),
+
+        "skip": SkipDirective(),
         # fmt: on
     }
